﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Cryptography.Pkcs;
using System.Web;
using Demo.Asymmetric.Payment.Data;

namespace Demo.Asymmetric.Payment {
    /// <summary>
    /// Summary description for InitPay1
    /// </summary>
    public class InitPay : IHttpHandler {

        public void ProcessRequest(HttpContext context) {
            // Read parameters
            var merchantId = int.Parse(context.Request.Form["MerchantId"]);
            var orderId = int.Parse(context.Request.Form["OrderId"]);
            var amount = decimal.Parse(context.Request.Form["Amount"], CultureInfo.InvariantCulture);
            var signatureString = context.Request.Form["Signature"];

            // Verify signature
            var signerThumbprint = AsymSigUtils.VerifySignature(signatureString,
                merchantId.ToString(),                          // 0 - Merchant ID
                orderId.ToString(),                             // 1 - Order ID
                amount.ToString(CultureInfo.InvariantCulture)); // 2 - Amount

            // Get merchant info from database
            using (var dc = new PaymentDbContext()) {
                var m = dc.Merchants.SingleOrDefault(x => x.MerchantId == merchantId);
                if (m == null) throw new Exception("Merchant not found!");
                if (!m.CertificateThumbprint.Equals(signerThumbprint, StringComparison.InvariantCultureIgnoreCase)) throw new Exception("Invalid or unknown signature!");

                // Check if we already have transaction with that order number
                if (m.Transactions.Any(x => x.MerchantOrderId == orderId)) throw new Exception("Duplicate order number");

                // Create transaction
                var t = new Transaction {
                    Amount = amount,
                    MerchantOrderId = orderId,
                    RequestSignature = signatureString
                };
                m.Transactions.Add(t);
                dc.SaveChanges();

                // Redirect to transaction confirmation
                context.Response.Redirect("~/ConfirmPay.aspx?TransactionId=" + t.TransactionId);
            }
        }

        public bool IsReusable {
            get { return true; }
        }
    }
}