﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Cryptography.Pkcs;
using System.Security.Cryptography.X509Certificates;
using System.Web;
using Demo.Asymmetric.Payment.Data;

namespace Demo.Asymmetric.Payment {
    /// <summary>
    /// Summary description for PaymentResponse
    /// </summary>
    public class PaymentResponse : IHttpHandler {

        public void ProcessRequest(HttpContext context) {
            using (var dc = new PaymentDbContext()) {
                // Get order from database
                var transactionId = int.Parse(context.Request.QueryString["TransactionId"]);
                var t = dc.Transactions.SingleOrDefault(x => x.TransactionId == transactionId && x.Status.HasValue);
                if (t == null) throw new Exception("Transaction not found");

                // Prepare data for signing
                var signature = AsymSigUtils.ComputeSignature(context.Server.MapPath("~/App_Data/payment.pfx"),
                    t.Status.Value ? "1" : "0",     // 0 - Status
                    t.MerchantOrderId.ToString());  // 1 - Order ID


                // Generate HTML form
                var html = string.Format(Properties.Resources.PaymentResponseTemplate,
                    t.Merchant.CallbackUrl,     // 0 - action
                    t.Status.Value ? "1" : "0", // 1 - status
                    t.MerchantOrderId,          // 1 - Order ID
                    signature);                 // 3 - signature
                context.Response.ContentType = "text/html";
                context.Response.Write(html);
            }
        }

        public bool IsReusable {
            get { return true; }
        }

    }
}