﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.Cryptography.Pkcs;
using System.Security.Cryptography.X509Certificates;
using System.Web;

namespace Demo.Asymmetric.Shop {
    public static class AsymSigUtils {

        public static string ComputeSignature(string keyFile, params string[] signedItems) {
            if (keyFile == null) throw new ArgumentNullException("keyFile");
            if (string.IsNullOrWhiteSpace(keyFile)) throw new ArgumentException("Value cannot be empty or whitespace only string.", "keyFile");
            
            // Get signed data
            var signedString = string.Join("|", signedItems);
            var signedData = System.Text.Encoding.UTF8.GetBytes(signedString);

            // Get signing keys
            var cert = new X509Certificate2(keyFile);

            // Create PKCS#7 signature
            var cms = new SignedCms(new ContentInfo(signedData), true);
            cms.ComputeSignature(new CmsSigner(cert), true);
            var signatureData = cms.Encode();
            var signatureString = Convert.ToBase64String(signatureData);

            return signatureString;
        }

        public static string VerifySignature(string signatureString, params string[] signedItems) {
            if (signatureString == null) throw new ArgumentNullException("signature");
            if (string.IsNullOrWhiteSpace(signatureString)) throw new ArgumentException("Value cannot be empty or whitespace only string.", "signature");
            
            // Get signed data
            var signedString = string.Join("|", signedItems);
            var signedData = System.Text.Encoding.UTF8.GetBytes(signedString);

            // Verify PKCS#7 package
            try {
                var signatureData = Convert.FromBase64String(signatureString);
                var cms = new SignedCms(new ContentInfo(signedData), true);
                cms.Decode(signatureData);
                cms.CheckSignature(true);

                // Get first signer certificate thumbprint
                return cms.SignerInfos[0].Certificate.Thumbprint;
            }
            catch (Exception) {
                return null;
            }
        }

    }
}