﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography.Pkcs;
using System.Web;
using Demo.Asymmetric.Shop.Data;
using System.Text;
using System.Configuration;

namespace Demo.Asymmetric.Shop {
    public class PaymentRequest : IHttpHandler {

        public void ProcessRequest(HttpContext context) {
            using (var dc = new ShopDbContext()) {
                // Get order from database
                var orderId = int.Parse(context.Request.QueryString["OrderId"]);
                var o = dc.Orders.SingleOrDefault(x => x.OrderId == orderId);
                if (o == null) throw new Exception("Order not found");

                // Get signature
                var signature = AsymSigUtils.ComputeSignature(context.Server.MapPath("~/App_Data/shop.pfx"),
                    "1",                                                // 0 - Merchant ID
                    orderId.ToString(),                                 // 1 - Order ID
                    o.Amount.ToString(CultureInfo.InvariantCulture));   // 2 - Amount

                // Generate HTML form
                var html = string.Format(Properties.Resources.PaymentRequestTemplate,
                    ConfigurationManager.AppSettings["PaymentGatewayBaseUrl"],  // 0 - action
                    "1",                                                        // 1 - Merchant ID
                    orderId.ToString(),                                         // 2 - Order ID
                    o.Amount.ToString(CultureInfo.InvariantCulture),            // 3 - Amount
                    signature);                                                 // 4 - signature
                context.Response.ContentType = "text/html";
                context.Response.Write(html);
            }
        }

        public bool IsReusable {
            get { return true; }
        }

    }
}