﻿using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Xml;
using Demo.Symmetric.Payment.Data;

namespace Demo.Symmetric.Payment {
    public partial class InitPay : System.Web.UI.Page {
        private int merchantId, orderId;
        private decimal amount;

        protected void Page_Load(object sender, EventArgs e) {
            // Read parameters
            this.merchantId = int.Parse(this.Request.QueryString["MerchantId"]);
            this.orderId = int.Parse(this.Request.QueryString["OrderId"]);
            this.amount = decimal.Parse(this.Request.QueryString["Amount"], CultureInfo.InvariantCulture);

            using (var dc = new PaymentDbContext()) {
                // Get merchant info from database
                var m = dc.Merchants.SingleOrDefault(x => x.MerchantId == this.merchantId);
                if (m == null) throw new Exception("Merchant not found!");

                // Verify digital signature on parameters
                var correctSig = SymSigUtils.ComputeSignature(m.RequestKey,
                    merchantId.ToString(),
                    orderId.ToString(),
                    amount.ToString(CultureInfo.InvariantCulture));
                if (!correctSig.Equals(Request.QueryString["Signature"], StringComparison.Ordinal)) throw new Exception("Invalid digital signature");

                // Check if we already have order with that number
                if (m.Transactions.Any(x => x.MerchantOrderId == orderId)) throw new Exception("Duplicate order");

                // Show message
                this.LiteralMessage.Text = string.Format(this.LiteralMessage.Text,
                    amount,         // 0
                    m.CompanyName,  // 1
                    orderId);       // 2
            }
        }

        protected void ButtonYes_Click(object sender, EventArgs e) {
            this.ProcessPayment(true);
        }

        protected void ButtonNo_Click(object sender, EventArgs e) {
            this.ProcessPayment(false);
        }

        // Helper methods

        private void ProcessPayment(bool success) {
            using (var dc = new PaymentDbContext()) {
                // Get merchant from database
                var m = dc.Merchants.Single(x => x.MerchantId == this.merchantId);

                // Save order to database
                m.Transactions.Add(new Transaction {
                    MerchantOrderId = this.orderId,
                    Amount = this.amount,
                    Status = success
                });
                dc.SaveChanges();

                // Compute signature
                var sig = SymSigUtils.ComputeSignature(m.ResponseKey,
                    success ? "1" : "0",        // status
                    this.orderId.ToString());   // merchant order id

                // Redirect to merchant
                var url = string.Format(m.CallbackUrl,
                    success ? "1" : "0",    // 0
                    this.orderId,           // 1
                    sig);                   // 2
                this.Response.Redirect(url);
            }
        }

    }
}