﻿using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Configuration;
using System.Globalization;
using System.Linq;
using System.Web;

namespace Demo.Symmetric.Shop.Data {
    public class Order {

        // Properties

        [Key]
        public int OrderId { get; set; }

        public DateTime DateCreated { get; set; }

        [MaxLength(100)]
        public string Description { get; set; }

        public decimal Amount { get; set; }

        public bool PaidFor { get; set; }

        // Constructor

        public Order() {
            this.DateCreated = DateTime.Now;
        }

        // Other methods

        public string GetPaymentUrl() {
            // Get key and compute signature
            var key = Convert.FromBase64String(ConfigurationManager.AppSettings["RequestKey"]);
            var sig = SymSigUtils.ComputeSignature(key, 
                "1",                                                    // Merchant ID
                this.OrderId.ToString(),                                // Order ID
                this.Amount.ToString(CultureInfo.InvariantCulture));    // Amount

            // Create URL
            var url = ConfigurationManager.AppSettings["PaymentGatewayBaseUrl"];
            return string.Format(url,
                this.OrderId,                                       // 0
                this.Amount.ToString(CultureInfo.InvariantCulture), // 1
                sig);                                               // 2
        }

    }
}